import json

import pytest
import responses

import datarobot.client
from datarobot.client import get_client

import mock


@pytest.yield_fixture
def client_version_mock():
    with mock.patch('datarobot.client._get_client_version') as mv:
        yield mv


@pytest.fixture
def version_endpoint(default_config_endpoint):
    return default_config_endpoint + '/version/'


class TestVersionCheck(object):

    @pytest.fixture
    def server_version_string(self):
            yield '2.3.0'

    @pytest.fixture(autouse=True)
    def server_version_response(self, version_endpoint, server_version_string):
        responses.add(
            responses.GET,
            version_endpoint,
            status=200,
            body=json.dumps({'major': '2', 'minor': '3', 'versionString': server_version_string})
        )

    @pytest.yield_fixture(autouse=True)
    def no_global_client(self, default_config):
        with mock.patch('datarobot.client._global_client', None) as gc:
            yield gc

    @responses.activate
    @pytest.mark.usefixtures('known_warning')
    def test_major_incompat(self, client_version_mock):
        client_version_mock.return_value = '3.0.0'
        client = get_client()
        assert client is None
        assert datarobot.client._global_client is None

    @responses.activate
    @pytest.mark.usefixtures('known_warning')
    def test_minor_incompat(self, client_version_mock):
        client_version_mock.return_value = '2.4.0'
        client = get_client()
        assert client is not None
        assert datarobot.client._global_client is client

    @responses.activate
    @pytest.mark.parametrize('server_version_string', ['2.4.0'])
    def test_server_can_be_ahead_of_client(self, client_version_mock):
        client_version_mock.return_value = '2.3.0'
        client = get_client()
        assert client is not None
        assert datarobot.client._global_client is client


class TestServerVersion(object):

    @responses.activate
    @pytest.mark.usefixtures('default_config', 'known_warning')
    def test_version_does_not_exist(self, version_endpoint):
        """This situation happens if a user is using a client that is new
        enough to check for version on a server that doesn't have an endpoint
        to supply that version
        """
        responses.add(
            responses.GET,
            version_endpoint,
            status=404,
            body=''
        )

        get_client()

    @responses.activate
    @pytest.mark.usefixtures('default_config')
    def test_version_does_exist(self, version_endpoint, client_version_mock):
        client_version_mock.return_value = '2.4.0'

        responses.add(
            responses.GET,
            version_endpoint,
            status=200,
            body=json.dumps({'major': '2', 'minor': '4', 'versionString': '2.4.0'})
        )

        get_client()


class TestClientServerEndpointProtocol(object):

    @responses.activate
    @pytest.mark.parametrize('default_config_endpoint, expected_endpoint', [
        pytest.param('https://user_config.com', 'https://user_config.com', id='https'),
        pytest.param('http://user_config.com', 'http://user_config.com', id='http'),
    ])
    @pytest.mark.usefixtures('default_config', 'known_warning')
    def test_correct_configuration_no_changes(self, version_endpoint, expected_endpoint):
        responses.add(
            responses.GET,
            version_endpoint,
            status=200,
            body=json.dumps({'major': '2', 'minor': '4', 'versionString': '2.4.0'})
        )

        client = get_client()
        assert client.endpoint == expected_endpoint

    @responses.activate
    @pytest.mark.parametrize('default_config_endpoint', ['http://user.config.com'])
    @pytest.mark.usefixtures('default_config', 'known_warning')
    def test_http_https_protocol_missmatch(self, version_endpoint, default_config_endpoint):
        https_version_endpoint = version_endpoint.replace('http://', 'https://')
        responses.add(
            responses.GET,
            version_endpoint,
            status=301,
            headers={'Location': https_version_endpoint}
        )
        responses.add(
            responses.GET,
            https_version_endpoint,
            status=200,
            body=json.dumps({'major': '2', 'minor': '4', 'versionString': '2.4.0'}),
        )
        client = get_client()
        assert client.endpoint.startswith('https://')



import trafaret as t

from datarobot.models.api_object import APIObject
from datarobot.utils import encode_utf8_if_py2


class LiftChart(APIObject):
    """ Lift chart data for model.

    Notes
    -----
    ``LiftChartBin`` is a dict containing the following:

        * ``actual`` (float) Sum of actual target values in bin
        * ``predicted`` (float) Sum of predicted target values in bin
        * ``bin_weight`` (float) The weight of the bin. For weighted projects, it is the sum of \
          the weights of the rows in the bin. For unweighted projects, it is the number of rows in \
          the bin.

    Attributes
    ----------
    source : str
        Lift chart data source. Can be 'validation', 'crossValidation' or 'holdout'.
    bins : list of dict
        List of dicts with schema described as ``LiftChartBin`` above.
    source_model_id : str
        ID of the model this lift chart represents; in some cases,
        insights from the parent of a frozen model may be used
    """
    _converter = t.Dict({
        t.Key('source'): t.String,
        t.Key('bins'): t.List(t.Dict({
            t.Key('actual'): t.Float,
            t.Key('predicted'): t.Float,
            t.Key('bin_weight'): t.Float
        }).ignore_extra('*')),
        t.Key('source_model_id'): t.String
    }).ignore_extra('*')

    def __init__(self, source, bins, source_model_id):
        self.source = source
        self.bins = bins
        self.source_model_id = source_model_id

    def __repr__(self):
        return encode_utf8_if_py2(u'LiftChart({})'.format(self.source))



import responses
import pytest

from datarobot.models import ComplianceDocTemplate


@pytest.fixture
def templates_list_response():
    return [
        {
            'id': '5bbdf361962d7435e2c447fc',
            'name': 'template #1',
            'creatorId': '5a421957aadebb4f0df2f096',
            'creatorUsername': 'admin@datarobot.com',
            'orgId': '5a421957aadebb4f0df2f091',
            'sections': [
                {'contentId': u'HOW_TO_USE',
                 'sections': [],
                 'title': u'How To Use This Document',
                 'type': u'datarobot'},
                {'title': u'Table of Contents', 'type': u'table_of_contents'},
                {'contentId': u'PREFACE',
                 'sections': [],
                 'title': u'DataRobot Model Development Documentation',
                 'type': u'datarobot'},
                {'contentId': u'EXECUTIVE_SUMMARY_AND_MODEL_OVERVIEW',
                 'sections': [],
                 'title': u'Exec Summary And Model Overview',
                 'type': u'datarobot'}
            ]
        },
        {
            'id': '5bbdf361962d7435e2c447fd',
            'name': 'template #2',
            'creatorId': '5a421957aadebb4f0df2f096',
            'creatorUsername': 'admin@datarobot.com',
            'orgId': '5a421957aadebb4f0df2f091',
            'sections': [
                {'contentId': u'HOW_TO_USE',
                 'sections': [],
                 'title': u'How To Use This Document',
                 'type': u'datarobot'},
                {'title': u'Table of Contents', 'type': u'table_of_contents'},
                {'contentId': u'PREFACE',
                 'sections': [],
                 'title': u'DataRobot Model Development Documentation',
                 'type': u'datarobot'},
                {'contentId': u'EXECUTIVE_SUMMARY_AND_MODEL_OVERVIEW',
                 'sections': [],
                 'title': u'Exec Summary And Model Overview',
                 'type': u'datarobot'}
            ]
        }
    ]


@responses.activate
@pytest.mark.usefixtures('client')
def test_list(templates_list_response):
    responses.add(
        responses.GET,
        'https://host_name.com/complianceDocTemplates/',
        json={
            'data': templates_list_response,
            'next': None,
            'previous': None,
            'count': len(templates_list_response)
        }
    )

    templates = ComplianceDocTemplate.list()

    for template, server_data in zip(templates, templates_list_response):
        assert template.id == server_data['id']
        assert template.name == server_data['name']
        assert template.creator_id == server_data['creatorId']
        assert template.creator_username == server_data['creatorUsername']
        assert template.org_id == server_data['orgId']
        assert template.sections is not None


@responses.activate
@pytest.mark.usefixtures('client')
def test_get(templates_list_response):
    server_data = templates_list_response[0]
    responses.add(
        responses.GET,
        'https://host_name.com/complianceDocTemplates/{}/'.format(server_data['id']),
        json=server_data
    )
    template = ComplianceDocTemplate.get(server_data['id'])

    assert template.id == server_data['id']
    assert template.name == server_data['name']
    assert template.creator_id == server_data['creatorId']
    assert template.creator_username == server_data['creatorUsername']
    assert template.org_id == server_data['orgId']
    assert template.sections is not None


@responses.activate
@pytest.mark.usefixtures('client')
def test_get_no_org_id(templates_list_response):
    server_data = dict(templates_list_response[0], orgId=None)
    responses.add(
        responses.GET,
        'https://host_name.com/complianceDocTemplates/{}/'.format(server_data['id']),
        json=server_data
    )
    template = ComplianceDocTemplate.get(server_data['id'])

    assert template.id == server_data['id']
    assert template.name == server_data['name']
    assert template.creator_id == server_data['creatorId']
    assert template.creator_username == server_data['creatorUsername']
    assert template.org_id is None
    assert template.sections is not None


@responses.activate
@pytest.mark.usefixtures('client')
def test_update(templates_list_response):
    server_data = templates_list_response[0]
    responses.add(
        responses.GET,
        'https://host_name.com/complianceDocTemplates/{}/'.format(server_data['id']),
        json=server_data
    )
    responses.add(
        responses.PATCH,
        'https://host_name.com/complianceDocTemplates/{}/'.format(server_data['id']),
        body='',
        status=204
    )
    template = ComplianceDocTemplate.get(server_data['id'])
    template.update(name='updated name')

    assert template.name == 'updated name'


@responses.activate
@pytest.mark.usefixtures('client')
def test_create(templates_list_response):
    data = templates_list_response[0]
    responses.add(
        responses.POST,
        'https://host_name.com/complianceDocTemplates/',
        body='',
        status=201,
        adding_headers={
            'Location': 'https://host_name.com/complianceDocTemplates/{}/'.format(data['id'])
        }
    )
    responses.add(
        responses.GET,
        'https://host_name.com/complianceDocTemplates/{}/'.format(data['id']),
        json=data,
    )
    template = ComplianceDocTemplate.create(
        name=data['name'],
        sections=data['sections']
    )
    assert template.id == data['id']
    assert template.name == data['name']
    assert template.creator_id == data['creatorId']
    assert template.creator_username == data['creatorUsername']
    assert template.org_id == data['orgId']
    assert template.sections is not None


@responses.activate
@pytest.mark.usefixtures('client')
def test_get_default(templates_list_response):
    server_data = templates_list_response[0]
    responses.add(
        responses.GET,
        'https://host_name.com/complianceDocTemplates/default/',
        json=server_data
    )
    template = ComplianceDocTemplate.get_default()
    assert template.id is None
    assert template.creator_id is None
    assert template.creator_username is None
    assert template.org_id is None
    assert template.sections is not None
    assert template.name == 'default'


@responses.activate
@pytest.mark.usefixtures('client')
def test_sections_to_json_file(templates_list_response, tmpdir):
    server_data = templates_list_response[0]
    responses.add(
        responses.GET,
        'https://host_name.com/complianceDocTemplates/default/',
        json=server_data
    )
    responses.add(
        responses.POST,
        'https://host_name.com/complianceDocTemplates/',
        body='',
        status=201,
        headers={'Location': 'https://host_name.com/complianceDocTemplates/{}/'.format(
            'new-template'
        )}
    )
    responses.add(
        responses.GET,
        'https://host_name.com/complianceDocTemplates/new-template/',
        json=dict(server_data, name='new template')
    )
    template = ComplianceDocTemplate.get_default()
    tmpfile = tmpdir.mkdir('sections').join('sections.json')
    template.sections_to_json_file(path=tmpfile.strpath)
    assert tmpfile.exists()

    new_template = ComplianceDocTemplate.create_from_json_file(
        name='new template',
        path=tmpfile.strpath
    )
    assert new_template.name == 'new template'
    assert new_template.creator_id == server_data['creatorId']
    assert new_template.creator_username == server_data['creatorUsername']
    assert new_template.org_id == server_data['orgId']
    assert new_template.sections is not None



import os
import json

import responses
import pytest

from datarobot.models import ComplianceDocumentation


@responses.activate
@pytest.mark.usefixtures('client')
def test_generate(project_id, one_model, job_id):
    responses.add(
        responses.POST,
        'https://host_name.com/projects/{}/models/{}/complianceDocs/'.format(
            project_id, one_model.id
        ),
        status=202,
        adding_headers={
            'Location': 'https://host_name.com/projects/{}/jobs/{}/'.format(project_id, job_id),
        },
    )
    stub_job_data = {
        'status': 'inprogress',
        'url': 'https://host_name.com/projects/{}/modelJobs/{}/'.format(project_id, job_id),
        'id': job_id,
        'jobType': 'model',
        'isBlocked': False,
        'projectId': project_id,
    }
    responses.add(
        responses.GET,
        'https://host_name.com/projects/{}/jobs/{}/'.format(project_id, job_id),
        status=200,
        body=json.dumps(stub_job_data),
        content_type='application/json'
    )
    doc = ComplianceDocumentation(project_id, one_model.id)
    job = doc.generate()

    assert job.id == int(job_id)
    assert len(responses.calls) == 2
    post_call, get_call = responses.calls
    assert post_call.request.body is None
    assert get_call.request.url.endswith('{}/'.format(job_id))


@responses.activate
@pytest.mark.usefixtures('client')
def test_download(project_id, one_model, tmpdir):
    url = 'https://host_name.com/projects/{}/models/{}/complianceDocs/'.format(
        project_id, one_model.id)
    responses.add(responses.GET,
                  url,
                  body='bytes and shit',
                  status=200,
                  match_querystring=True)
    doc = ComplianceDocumentation(project_id, one_model.id)
    localpath = tmpdir.join('temp.docx')
    doc.download(localpath.strpath)
    assert os.path.isfile(localpath.strpath)
    with open(localpath.strpath) as f:
        contents = f.read()
        assert contents == 'bytes and shit'
